#    software histogramming

loadsource hist

set frametitle "Software Histogramming"
set icontitle "Histogramming"


set SpectrumServer [inform sys inquire SpectrumServer]

set resources [list $SpectrumServer]
set numresources [llength $resources]

set resource $SpectrumServer


proc make-resource-list {} {}

proc execute-spectrum {n action} {
  global resource
  global debug

  set server $resource

  switch $action {
     create {
        set range [range-of $n]
        if {$debug} then {insert-debug "create spectrum :range $range :name '$n' "}
        set specname [SpectrumName $n]   ;#  apply alias feature
        set z [catch {eg "(spectrum :path '/$server' :range $range :name '$specname' !new)" } m]
#    Allow "Spectrum already exists" so that further spectra can be created
        if {$z == 0} then {inform sys map_needed}
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     delete {
        if {$debug} then {insert-debug "delete spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !delete)"} m]
# Allow "Pathname Invalid" so that further spectra can be deleted
        if {$z == 0} then {inform sys map_needed}     ;#  so that unmap occurs
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     clear {
        if {$debug} then {insert-debug "clear spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !zero)"} m]
     }
  }
  if {$z!=0} then {
    display-eg-error-report self $m "Attempting to " $action "spectrum" $n
    clear-footer
    error $m
  }
}


#    some procedures taken from SYS which implement the histogram alias feature 


if {![info exists XlatePres]}       {set XlatePres 0}

proc SpectrumName {n} {Give_Xlate $n}

proc Make_Xlate {n} {
   global env Xlate XlateR XlatePres

   if {[info exists Xlate]}  {unset Xlate}
   if {[info exists XlateR]} {unset XlateR}
   set XlatePres 0

     set z [catch {set directory [file join $env(EXPERIMENTBASE) Histconfigs]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        return
     }

     if {![file exists $directory]} then {
        set z [catch {:file mkdir $directory} m]
        if {$z!=0} then {midas-warning "Cannot create directory $directory\n $m"}
     }

   #  read name translation file

   set file [file join $directory [set n]_names]
   set z [catch {open $file r} f]
   if {$z != 0} then {
       set-footer "Histogram name translation file does not exist"
       set Names {}

       set z [catch {open $file w} f]
       if {$z != 0} then {midas-warning "Unable to write file $file\n$f"; return}
       close $f

   } else {
       clear-footer ""
       set Names [split [read $f] \n]
       close $f
   }
   
   foreach item $Names {
       if {$item == ""} {continue}
       if {[third $item] == 0} {continue}
       if {[string range [first $item] 0 0] == "#"} continue
       set Xlate([first $item])  [second $item]
       set XlateR([second $item])  [first $item]
       set XlatePres 1
   }
}

proc Give_Xlate {n} {Do_Xlate $n}
proc Give_XlateR {n} {Do_XlateR $n}

proc Do_Xlate {n} {
   global XlatePres Xlate

   if {$XlatePres == 0} {return $n}
   if {[info exists Xlate($n)]} {return $Xlate($n)} else {return $n}
}

proc Do_XlateR {n} {
   global XlatePres XlateR

   if {$XlatePres == 0} {return $n}
   if {[info exists XlateR($n)]} {return $XlateR($n)} else {return $n}
}
