
#    data acquisition support not yet in sys.common




lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "VME data source configuration"    "Configure_VME_Data_Sources"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_VME_Data"]
lappend actionlist [list "Print Hardware Configuration"  "PrintConfig"]



proc create_spectra {} {
   global SpectrumServer HistMode EGreport Options
   global Supported_Modules

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1 &&  $Options(Histogram.$module) == 1} then {
           create_[set module]_spectra
       }
   }

   global 2Dlist 2Dpresent
   if {[llength $2Dlist] == 0} {
       if {$2Dpresent == 0} {2D_Histogram_Configuration; close-frame "[self frame-name].2dhistograms"}
   }
   Create_2D_Histograms

   set EGreport $EGreport_SAVE

   clear-footer
}

proc define_VME_addresses {} {
   global Supported_Modules Options

#    load hardware configuration into readout software

     ClearConfig

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1} then {define_[set module]_config}
   }

#    then the acquisition control module

     define_sac_config
}

proc click-vme {action args} {
    global VMEPresent VMEEnable
   global Supported_Modules 

  if {$VMEPresent == 0} then {return}
  if {$VMEEnable == 0} then {:modify VME :value -1; return}

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }

   set-footer "sending $p to VME control task"
   set-busy
   if {[llength $args] == 0} then {
       set z [catch {send vmecontrol $p} m]
   } else {
       set z [catch {send vmecontrol $p [first $args]} m]
   }
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   if {$action == 3} then {define_VME_addresses}

   if {[lsearch $Supported_Modules V767] >= 0} {do_V767_setup $action}
   if {[lsearch $Supported_Modules V1190] >= 0} {do_V1190_setup  $action}
   if {[lsearch $Supported_Modules V1290] >= 0} {do_V1290_setup $action}

   redisplay-vme
   update_button_states
}

proc redisplay-vme {} {
    global RegisterServer
    global VMEEnable vmestate EGallow

    if {$VMEEnable == 0} then {return}

    set RegisterServer [inform vmecontrol inquire Role]

    set xvme260pres [inform vmecontrol inquire xvme260count]
    if {[string length $xvme260pres] > 2} {set xvme260pres 0}
    if {$xvme260pres != 0} {
       set vl [read-register xvme260#1.PortA]
           if {$vl == "0x0000"} {:modify VME :value 1} else {:modify VME :value 0}
           return
    }

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} then {set vmestate -1; :modify VME :value $vmestate; return}

    set EGallow 0x10004

    set vl [read-register sac#1.Enable]
    if {$vl == 0x10004} then {
       midas-warning "Required hardware register is not known\n\
         Check your hardware configuration. Possibly a SETUP is needed"
       set vmestate -1
    } else {
       set vmestate [expr $vl & 1]
    }

    :modify VME :value $vmestate

    set EGallow 0xffffffff
}

proc Stop_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

    set xvme260pres [inform vmecontrol inquire xvme260count]
    if {[string length $xvme260pres] > 2} {set xvme260pres 0}
    if {$xvme260pres != 0} {
       write-registers xvme260#1.PortA 0
    }


    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}
    if {$sacpres != 0} {
        write-registers sac#1.STR 0
    }
}

proc Go_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

    set xvme260pres [inform vmecontrol inquire xvme260count]
    if {[string length $xvme260pres] > 2} {set xvme260pres 0}
    if {$xvme260pres != 0} {
       write-register xvme260#1.PortA -1
    }

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} {return}     ;#  no SAC is possible

    set maxmodule [inform vmecontrol inquire s9418count]
    if {[string length $maxmodule] > 2} {set maxmodule 0}
       
    if {$maxmodule == 0} then {
        write-registers sac#1.Mode 1   ;#   CAEN mode
    } else {
        write-registers sac#1.Mode 0   ;#   Silena 9418 mode (default)
    }

    write-registers sac#1.MON0 2
    write-registers sac#1.STR 9
    write-registers sac#1.ARM 0
}

proc Reset_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} {return}     ;#  no SAC is  present

    write-registers sac#1.EVCLR 0
}
